from mkdocs.plugins import BasePlugin, event_priority
import json
import os

class ExcludeStandaloneSnippetsPlugin(BasePlugin):
    """
    A MkDocs plugin that excludes standalone snippet pages not listed in the site's navigation from the search index.
    """
    
    def flatten_nav(self, nav_items, parent_dir=''):
        """
        Recursively flattens the navigation structure to a list of Markdown file paths.
        
        Args:
            nav_items (list): The navigation items to process, which can be a mix of dictionaries (for nested navigation) and strings (for direct links).
            parent_dir (str): The current parent directory path to prepend to nested navigation items, ensuring the full path is captured.
            
        Returns:
            list: A list of strings, where each string is a path to a Markdown file included in the site's navigation.
        """
        pages = []
        for item in nav_items:
            if isinstance(item, dict):  # Process nested navigation items
                for nested_items in item.values():
                    if isinstance(nested_items, list):
                        # Recurse into the list, carrying the current parent_dir
                        pages += self.flatten_nav(nested_items, parent_dir)
                    elif isinstance(nested_items, str) and nested_items.endswith('.md'):
                        # Add direct .md file references, prepending parent_dir if present
                        md_path = os.path.join(parent_dir, nested_items)
                        pages.append(md_path)
            elif isinstance(item, str) and item.endswith('.md'):  # Handle top-level .md files
                md_path = os.path.join(parent_dir, item)
                pages.append(md_path)
        return pages

    @event_priority(-100)  # Run this plugin's on_post_build event last
    def on_post_build(self, config, **kwargs):
        """
        The method called by MkDocs after the site has been built, to filter out standalone snippets from the search index.
        
        Args:
            config (dict): The MkDocs config object containing site configuration details.
        """
        search_index_path = os.path.join(config['site_dir'], 'search', 'search_index.json')
        
        with open(search_index_path, 'r') as file:
            search_index = json.load(file)
        
        # Generate a list of normalized navigation paths for comparison
        navigation_pages = [os.path.splitext(page)[0] for page in self.flatten_nav(config['nav'])]

        initial_count = len(search_index['docs'])

        filtered_docs = []
        for doc in search_index['docs']:
            # Normalize the document's location for comparison, removing localization and section identifiers
            parts = doc['location'].split('/')
            normalized_parts = [part for part in parts if not part.startswith('#') and part != 'en']
            normalized_doc_path = '/'.join(normalized_parts).rstrip('/').replace('.md', '')

            # Include the document if its normalized path matches any navigation path
            if normalized_doc_path in navigation_pages or any(normalized_doc_path.startswith(nav + '/') for nav in navigation_pages):
                filtered_docs.append(doc)
            # Optionally, for debugging purposes, you could uncomment the following lines to log
            # documents that are identified for exclusion. This can be helpful during development
            # and testing to verify that the plugin is correctly identifying documents to exclude.    
            # else:
               # print(f"Exclude_Snippets: Excluding from search index: {doc['location']}")

        final_count = len(filtered_docs)

        # Update the search index if any documents were excluded
        if initial_count != final_count:
            search_index['docs'] = filtered_docs
            with open(search_index_path, 'w') as file:
                json.dump(search_index, file)
            print("Exclude_Snippets: Successfully updated search_index.json with filtered documents.")

            # Optionally create a debug copy of the updated search index for comparison
            # debug_search_index_path = os.path.join(config['site_dir'], 'search', 'search_index_debug.json')
            # with open(debug_search_index_path, 'w') as debug_file:
            #     json.dump(search_index, debug_file)
            # print("Exclude_Snippets: Created a debug copy of the updated search_index.json at 'search/search_index_debug.json'.")


    def is_standalone_snippet(self, doc, navigation_pages):
        """
        Determines if a document should be excluded from the search index.
        
        This method checks if the normalized path of a document is not present in the list
        of navigation pages. The normalization process involves removing the '.md' extension
        from the document's location for a consistent comparison with the navigation paths.
        
        Args:
            doc (dict): A dictionary representing a document in the search index, where
                        'location' is a key pointing to the document's path.
            navigation_pages (list): A list of strings representing the paths of documents
                                    included in the site's navigation, as normalized by
                                    the flatten_nav method.
        
        Returns:
            bool: True if the document is not found in the navigation pages and should be
                excluded from the search index; False otherwise.
        """
        # Normalize the document location by removing the '.md' extension. This step is crucial
        # for ensuring that the document's path can be directly compared against the list of
        # navigation paths, which have been similarly normalized.
        normalized_doc_path = doc['location'].rstrip('.md')

        # Check if the normalized document path exists within the list of navigation pages.
        # If the path is not found, the document is considered a standalone snippet that is not
        # directly accessible through the site's navigation and thus should be excluded from
        # the search index to avoid leading users to potentially orphaned or unintended pages.
        is_excluded = normalized_doc_path not in navigation_pages

        return is_excluded



